#ifndef T6963_H
#define T6963_H
#include <string.h>
#include "types.h"
#include "lpc21xx.h"
#include "config.h"

#define LCD_BUSPIN1 16 	/* Bus on Pins P1.16->P1.23 */

#define LCD_DATA    (uint32_t)(0xff<<LCD_BUSPIN1)
#define LCD_DATABUS	0x00FF0000

#define LCD_RST		(uint32_t)(1<<28)  /* Reset on P1.28 */
#define LCD_CE		(uint32_t)(1<<26)  /* Chip enable on P1.26  */
#define LCD_WR		(uint32_t)(1<<24)  /* Write on P1.24 */
#define LCD_CD		(uint32_t)(1<<27)  /* Data/Instr. P1.27 */
#define LCD_RD		(uint32_t)(1<<25)  /* Read  on P1.25  */

#define LCD_CONTROL	(uint32_t)(LCD_RD|LCD_CE|LCD_WR|LCD_CD|LCD_RST)

#define DATAOUT() ( IO1DIR |= (LCD_DATABUS) )	/* Databus all output */
#define DATAIN()  ( IO1DIR &= ~(LCD_DATABUS) )  /* Databus all in  */

#define FONTWIDTH 8
#define TXT 0//used to select which
#define GFX 1//screen to clear
#define XBYTES    30
#define YBYTES    16
#define XDOTS   240
#define YDOTS   128
#define GFXSIZE (XBYTES*YBYTES*8)
#define PIXELSET 1
#define PIXELCLR 0

#define GA 30// number of bytes per line
#define TA 30// number of character per line
//#define GPAGE1 	0x0000//3840bytes
//#define GPAGE2 	0x0F01//3840bytes
//#define GPAGE3 	0x1E02//3840bytes
//#define GPAGE4 	0x3C04//3840bytes
//#define GPAGE5 	0x4B05//3840bytes
//#define TATTR 	0x5A06//640bytes
//#define TPAGE1	0x5BE7//640bytes
//#define TPAGE2	0x5DC8//640bytes
//#define TPAGE3	0x5FA9//640bytes
//#define TPAGE4	0x618A//640bytes
//#define TPAGE5	0x636B//640bytes
//#define CGRAM	0x645C//atleast 2048bytes
//#define CGOFFSET  0x000C
#define GPAGE1 	0x0000//3840bytes
#define GPAGE2 	0x0F00//3840bytes
#define GPAGE3 	0x1E00//3840bytes
#define GPAGE4 	0x2D00//3840bytes
#define GPAGE5 	0x3C00//3840bytes
#define TATTR 	0x4B00//3840bytes text attributes in graphic ram
#define TPAGE1 	0x5A00//640bytes
#define TPAGE2	0x5C80//640bytes
#define TPAGE3	0x5F00//640bytes
#define TPAGE4	0x6180//640bytes
#define TPAGE5	0x6400//640bytes
#define TPAGE6	0x6680//640bytes
#define CGRAM	0x6900//atleast 2048bytes
#define CGOFFSET  0x000D
// #define TATTR  0x4CE6
// #define TPAGE1 0x4EC7
// #define TPAGE2 0x50A8
// #define TPAGE3 0x5289

#define INT_CG_MODE         0x80    //0b10000xxx, Internal CG ROM mode
#define EXT_CG_MODE         0x88    //0b10001xxx, External CG RAM mode
#define OR_MODE             0x80    //0b1000x000, OR mode, internal CG ROM mode
#define XOR_MODE            0x81    //0b1000x001, XOR mode, internal CG ROM mode
#define AND_MODE            0x83    //0b1000x011, AND mode, internal CG ROM mode
#define TEXT_ATTR_MODE      0x84    //0b1000x100, text attribute mode, internal CG ROM mode

#define DISPLAY_OFF         0x90    //0b10010000, display off
#define CURSOR_ON_BLINK_OFF 0x92    //0b1001xx10, cursor on without blinking
#define CURSOR_BLINK_ON     0x93    //0b1001xx11, cursor on with blinking
#define TEXT_ON             0x94    //0b100101xx, text on, graphics off
#define GRFX_ON             0x98    //0b100110xx, text off, graphics on
#define TEXT_GRFX_ON        0x9C    //0b100111xx, text on, graphics on

#define LINE_8_CURSOR       0xA7    //8-line cursor mode
#define LINE_7_CURSOR		0xA6	//7-line cursor mode
#define LINE_6_CURSOR		0xA5	//6-line cursor mode
#define LINE_5_CURSOR		0xA4	//5-line cursor mode
#define LINE_4_CURSOR		0xA3	//4-line cursor mode
#define LINE_3_CURSOR		0xA2	//3-Line cursor mode
#define LINE_2_CURSOR		0xA1	//2-Line cursor mode
#define LINE_1_CURSOR       0xA0    //1-Line cursor mode

#define ATTR_NORMAL         0x00    //Normal Display
#define ATTR_REVERSE        0x05    //Reverse Display
#define ATTR_INHIBIT        0x03    //Inhibit Display
#define ATTR_BLINK          0x08    //Blinking of Normal Display
#define ATTR_BLINK_REVERSE  0x0D    //Blinking of Reverse Display
#define ATTR_BLINK_INHIBIT  0x0B    //Blinking of Inhibit Display

typedef struct{
        uint8_t width;
        uint8_t height;
        uint8_t data[];
}IMG;

uint16_t CurTxtPage, CurGfxPage;//current pages

void lcd_sendcmd(uint8_t dat);
void lcd_setaddr( uint16_t addr, uint8_t cmd );
uint8_t lcd_putc(char c);
uint8_t lcd_readbyte(void);
uint8_t lcd_autoread(void);
uint8_t lcd_readline(char*lcdline, uint8_t linenum);
void lcd_clearline(uint8_t linenum);
void lcd_setpixel(uint16_t col, uint16_t row, uint8_t show);
void lcd_clrpixel(uint16_t col, uint16_t row);
void lcd_gotoxy(uint8_t scrn, uint8_t x, uint8_t y);
void lcd_clr(uint8_t txtgfx);   //clear text or graphic area
void cfgGLCD(void);
void cls(void);
void cleargfx(void);
void cleartext(void);
void draw_msgbox(void);
void lcd_box(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t show);
void lcd_box_filled (uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, unsigned char show);
uint8_t lcd_drawbox(uint8_t xstart, uint8_t ystart, uint8_t width, uint8_t height, uint8_t fill);
uint8_t lcd_draw_img(uint8_t xstart, uint8_t ystart, IMG * img);
#endif
